package org.dromara.system.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import org.dromara.common.core.utils.MapstructUtils;
import org.dromara.common.core.utils.StringUtils;
import org.dromara.common.mybatis.core.page.TableDataInfo;
import org.dromara.common.mybatis.core.page.PageQuery;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.dromara.common.redis.utils.RedisUtils;
import org.dromara.system.domain.GameAthlete;
import org.dromara.system.domain.GameEvent;
import org.dromara.system.domain.constant.GameEventConstant;
import org.dromara.system.domain.vo.GameEventProjectVo;
import org.dromara.system.mapper.GameAthleteMapper;
import org.dromara.system.mapper.GameEventMapper;
import org.springframework.stereotype.Service;
import org.dromara.system.domain.bo.GameTeamBo;
import org.dromara.system.domain.vo.GameTeamVo;
import org.dromara.system.domain.GameTeam;
import org.dromara.system.mapper.GameTeamMapper;
import org.dromara.system.service.IGameTeamService;

import javax.swing.text.html.Option;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 参赛队伍Service业务层处理
 *
 * @author zlt
 * @date 2025-07-30
 */
@Slf4j
@RequiredArgsConstructor
@Service
public class GameTeamServiceImpl implements IGameTeamService {

    private final GameTeamMapper baseMapper;

    private final GameAthleteMapper gameAthleteMapper;

    private final GameEventMapper gameEventMapper;

    /**
     * 查询参赛队伍
     *
     * @param teamId 主键
     * @return 参赛队伍
     */
    @Override
    public GameTeamVo queryById(Long teamId) {
        return baseMapper.selectVoById(teamId);
    }

    /**
     * 分页查询参赛队伍列表
     *
     * @param bo        查询条件
     * @param pageQuery 分页参数
     * @return 参赛队伍分页列表
     */
    @Override
    public TableDataInfo<GameTeamVo> queryPageList(GameTeamBo bo, PageQuery pageQuery) {
        if (bo.getEventId() == null) {
            Object cacheObject = RedisUtils.getCacheObject(GameEventConstant.DEFAULT_EVENT_ID);
            if (cacheObject instanceof Integer) {
                bo.setEventId(((Integer) cacheObject).longValue());  // 显式转换为 Long 类型
            } else if (cacheObject instanceof Long) {
                bo.setEventId((Long) cacheObject);
            }
        }
        LambdaQueryWrapper<GameTeam> lqw = buildQueryWrapper(bo);
        Page<GameTeamVo> result = baseMapper.selectVoPage(pageQuery.build(), lqw);
        // 查询领队和队员列表
        result.getRecords().stream()
            .map(vo -> {
                // 处理运动员列表
                Optional.ofNullable(vo.getAthleteValue())
                    .filter(StringUtils::isNotBlank)
                    .ifPresent(athleteValue -> {
                        String[] projectIds = athleteValue.split(",");
                        List<GameAthlete> athletes = gameAthleteMapper.selectList(
                            Wrappers.<GameAthlete>lambdaQuery()
                                .in(GameAthlete::getAthleteId, projectIds)
                                .select(GameAthlete::getName));

                        vo.setAthleteValue(athletes.stream()
                            .map(GameAthlete::getName)
                            .filter(StringUtils::isNotBlank)
                            .collect(Collectors.joining(",")));
                    });
                // 处理领队
                Optional.ofNullable(vo.getLeader())
                    .filter(StringUtils::isNotBlank)
                    .ifPresent(leaderId -> {
                        GameAthlete athlete = gameAthleteMapper.selectOne(
                            Wrappers.<GameAthlete>lambdaQuery()
                                .eq(GameAthlete::getAthleteId, leaderId)
                                .select(GameAthlete::getName));
                        Optional.ofNullable(athlete)
                            .map(GameAthlete::getName)
                            .ifPresent(vo::setLeader);
                    });
                //处理赛事名称
                Optional.ofNullable(vo.getEventId())
                    .filter(ObjectUtil::isNotEmpty)
                    .ifPresent(eventId -> {
                        GameEvent gameEvent = gameEventMapper.selectOne(
                            Wrappers.<GameEvent>lambdaQuery()
                                .eq(GameEvent::getEventId, eventId)
                                .select(GameEvent::getEventName));
                        Optional.ofNullable(gameEvent)
                            .map(GameEvent::getEventName)
                            .ifPresent(vo::setEventName);
                    });
                return vo;
            })
            .collect(Collectors.toList());  // 收集结果
        return TableDataInfo.build(result);
    }

    /**
     * 查询符合条件的参赛队伍列表
     *
     * @param bo 查询条件
     * @return 参赛队伍列表
     */
    @Override
    public List<GameTeamVo> queryList(GameTeamBo bo) {
        if (bo.getEventId() == null) {
            Object cacheObject = RedisUtils.getCacheObject(GameEventConstant.DEFAULT_EVENT_ID);
            if (cacheObject instanceof Integer) {
                bo.setEventId(((Integer) cacheObject).longValue());  // 显式转换为 Long 类型
            } else if (cacheObject instanceof Long) {
                bo.setEventId((Long) cacheObject);
            }
        }
        LambdaQueryWrapper<GameTeam> lqw = buildQueryWrapper(bo);
        return baseMapper.selectVoList(lqw);
    }

    private LambdaQueryWrapper<GameTeam> buildQueryWrapper(GameTeamBo bo) {
        Map<String, Object> params = bo.getParams();
        LambdaQueryWrapper<GameTeam> lqw = Wrappers.lambdaQuery();
        lqw.orderByAsc(GameTeam::getTeamId);
        lqw.eq(bo.getEventId() != null, GameTeam::getEventId, bo.getEventId());

        Optional.ofNullable(bo.getEventName())
            .filter(StringUtils::isNotBlank)
            .ifPresent(eventName -> {
                List<GameEvent> gameEvents = gameEventMapper.selectList(
                    Wrappers.<GameEvent>lambdaQuery()
                        .like(GameEvent::getEventName, eventName)
                        .select(GameEvent::getEventId)
                );
                if (CollectionUtils.isNotEmpty(gameEvents)) {
                    Set<Long> set = gameEvents.stream()
                        .map(GameEvent::getEventId)
                        .collect(Collectors.toSet());
                    lqw.in(GameTeam::getEventId, set);
                } else {
                    lqw.apply("1=0");
                }
            });

        lqw.like(StringUtils.isNotBlank(bo.getTeamName()), GameTeam::getTeamName, bo.getTeamName());
        lqw.eq(StringUtils.isNotBlank(bo.getTeamCode()), GameTeam::getTeamCode, bo.getTeamCode());
        lqw.eq(StringUtils.isNotBlank(bo.getLeader()), GameTeam::getLeader, bo.getLeader());
        lqw.eq(StringUtils.isNotBlank(bo.getAthleteValue()), GameTeam::getAthleteValue, bo.getAthleteValue());
        lqw.eq(StringUtils.isNotBlank(bo.getProjectValue()), GameTeam::getProjectValue, bo.getProjectValue());
        lqw.eq(bo.getAthleteNum() != null, GameTeam::getAthleteNum, bo.getAthleteNum());
        lqw.eq(StringUtils.isNotBlank(bo.getNumberRange()), GameTeam::getNumberRange, bo.getNumberRange());
        lqw.eq(StringUtils.isNotBlank(bo.getTeamDescribe()), GameTeam::getTeamDescribe, bo.getTeamDescribe());
        lqw.eq(StringUtils.isNotBlank(bo.getStatus()), GameTeam::getStatus, bo.getStatus());
        return lqw;
    }

    /**
     * 新增参赛队伍
     *
     * @param bo 参赛队伍
     * @return 是否新增成功
     */
    @Override
    public Boolean insertByBo(GameTeamBo bo) {
        if (bo.getEventId() == null) {
            Object cacheObject = RedisUtils.getCacheObject(GameEventConstant.DEFAULT_EVENT_ID);
            if (cacheObject instanceof Integer) {
                bo.setEventId(((Integer) cacheObject).longValue());  // 显式转换为 Long 类型
            } else if (cacheObject instanceof Long) {
                bo.setEventId((Long) cacheObject);
            }
        }
        GameTeam add = MapstructUtils.convert(bo, GameTeam.class);
        validEntityBeforeSave(add);
        boolean flag = baseMapper.insert(add) > 0;
        if (flag) {
            bo.setTeamId(add.getTeamId());
        }
        return flag;
    }

    /**
     * 修改参赛队伍
     *
     * @param bo 参赛队伍
     * @return 是否修改成功
     */
    @Override
    public Boolean updateByBo(GameTeamBo bo) {
        if (bo.getEventId() == null) {
            Object cacheObject = RedisUtils.getCacheObject(GameEventConstant.DEFAULT_EVENT_ID);
            if (cacheObject instanceof Integer) {
                bo.setEventId(((Integer) cacheObject).longValue());  // 显式转换为 Long 类型
            } else if (cacheObject instanceof Long) {
                bo.setEventId((Long) cacheObject);
            }
        }
        GameTeam update = MapstructUtils.convert(bo, GameTeam.class);
        validEntityBeforeSave(update);
        return baseMapper.updateById(update) > 0;
    }

    /**
     * 保存前的数据校验
     */
    private void validEntityBeforeSave(GameTeam entity) {
        //TODO 做一些数据校验,如唯一约束
    }

    /**
     * 校验并批量删除参赛队伍信息
     *
     * @param ids     待删除的主键集合
     * @param isValid 是否进行有效性校验
     * @return 是否删除成功
     */
    @Override
    public Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid) {
        if (isValid) {
            //TODO 做一些业务上的校验,判断是否需要校验
        }
        return baseMapper.deleteByIds(ids) > 0;
    }

    /**
     * 批量保存参赛队伍信息
     *
     * @param list 参赛队伍信息
     */
    @Override
    public Boolean saveBatch(List<GameTeam> list) {
        return baseMapper.insertBatch(list);
    }

    /**
     * 获取参赛队伍数量
     *
     * @return
     */
    @Override
    public Long countTeam() {
        return this.baseMapper.selectCount(Wrappers.lambdaQuery(GameTeam.class));
    }
}
